%CIRCUITBOARD   Demo to find horizontal, vertical and diagonal lines

% Author:
% Cris Luengo, Centre for Image Analysis, Uppsala, Sweden.
% Copyright 2008-2009.
%
% This code may be used freely for research purposes only. Do not
% copy or distribute without the prior written consent by the author.

set(0,'defaultfigureinvertHardcopy','off')

a = readim('circuitboard.jpg');
a = a{2};
a = dip_image(rotation(dip_image(a,'sfloat'),0.08),'uint8');
a = a(60:1020,80:760);
b = a-opening(a,13);
b = dip_image(b,'uint8');

dipshow(0,a(130+(1:620),15+(1:620)));
text(600,600,'(a)','fontsize',34,'horizontalalignment','right','verticalalignment','bottom','margin',5,'background','black','color','white')
print -deps2c -r0 circuitboard_input.eps

%% PATH OPENINGS

c = directedpathopening(b,[],[50,0],0,1);
d = directedpathopening(b,[],[0,50],0,1);
e = directedpathopening(b,[],[50,50],0,1);
f = directedpathopening(b,[],[50,-50],0,1);
e = max(e,f);

dipshow(0,e(130+(1:620),15+(1:620)),[0,192]);
text(600,600,'(b)','fontsize',34,'horizontalalignment','right','verticalalignment','bottom','margin',5,'background','black','color','white')
print -deps2c -r0 circuitboard_diag.eps
dipshow(0,c(130+(1:620),15+(1:620)),[0,192]);
text(600,600,'(c)','fontsize',34,'horizontalalignment','right','verticalalignment','bottom','margin',5,'background','black','color','white')
print -deps2c -r0 circuitboard_horz.eps
dipshow(0,d(130+(1:620),15+(1:620)),[0,192]);
text(600,600,'(d)','fontsize',34,'horizontalalignment','right','verticalalignment','bottom','margin',5,'background','black','color','white')
print -deps2c -r0 circuitboard_vert.eps

if 0
   g = joinchannels('rgb',c,d,e);
   g = colorspace(g,'hcv');
   g{3} = a*0.8;
   g = colorspace(g,'rgb');
   g = dip_image(g,'uint8');
end

%% STRAIGHT LINE OPENINGS

c = opening(b,[50,0],'rectangular');
d = opening(b,[0,50],'rectangular');
sz = imsize(b);
b = rotation(dip_image(b,'sfloat'),pi/4);
e = opening(b,[50,0],'rectangular');
f = opening(b,[0,50],'rectangular');
e = max(e,f);
e = rotation(e,-pi/4);
sz2 = imsize(e);
left = floor((sz2-sz)/2);
right = left+sz-1;
e = e(left(1):right(1),left(2):right(2));
e = dip_image(e,'uint8');

dipshow(0,c(130+(1:620),15+(1:620)),[0,192]);
text(600,600,'(e)','fontsize',34,'horizontalalignment','right','verticalalignment','bottom','margin',5,'background','black','color','white')
print -deps2c -r0 circuitboard_horz_X.eps
dipshow(0,d(130+(1:620),15+(1:620)),[0,192]);
text(600,600,'(f)','fontsize',34,'horizontalalignment','right','verticalalignment','bottom','margin',5,'background','black','color','white')
print -deps2c -r0 circuitboard_vert_X.eps



if 0 % For comparison, some other methods

%% STRUCTURE TENSOR

b = a-gaussf(a,5);
[orient,energy,anis,l1,l2] = structuretensor(b,1,3,{'orientation','energy','anisotropy','l1','l2'});
st_c = abs(mod(orient,pi)-pi/2);
st_c = pi/4 - st_c;
st_c(st_c<0) = 0;
st_c = clip( st_c * anis * b ,0,1e9); % close to selected angle, high anisotropy, high grey-value
st_d = abs(mod(orient+pi/2,pi)-pi/2);
st_d = pi/4 - st_d;
st_d(st_d<0) = 0;
st_d = clip( st_d * anis * b ,0,1e9);

%% STEERABLE FILTERS (sort of)

sf_c = clip( -derivative(a,[5,1],[0,2],'gaussfir') ,0,1e9);
sf_d = clip( -derivative(a,[1,5],[2,0],'gaussfir') ,0,1e9);

end
