%GRANULOMETRY3D   Demo how to compute a length distribution in 3D
%   NOTE: private data is used here, which I cannot currently
%   distribute. Feel free to replace the first command below
%   (the LOAD command) with something that will read one of your
%   own 3D images.

% Author:
% Cris Luengo, Centre for Image Analysis, Uppsala, Sweden.
% Copyright 2008-2009.
%
% This code may be used freely for research purposes only. Do not
% copy or distribute without the prior written consent by the author.


% Read in volume & downsample
load /data/fiber/WoodWisdom/SLS_FEB09/disk1/T11003_/rec_16bit_cbazp/vol.8bit.mat
vol = dip_image(vol);
vol = vol(0:2:end,0:2:end,150:2:end-150);
vol_orig = vol;
vol = bilateralf(vol,1,10,3,'xysep');
vol = dip_image(vol,'uint8');
vol_smooth = vol;

% Semi-binarize (the triangle threshold selection)
[h,b] = diphist(vol,[]);
[t,t] = max(h); % the top bin
r = length(h);  % the rightmost bin
v1 = [t-r,h(t)-h(r)];
v1 = v1/norm(v1);
f = t:r;
v2 = [f'-r,h(f)'-h(r)];
dis = abs(v1(1).*v2(:,2)-v1(2).*v2(:,1));
[fm,fm] = max(dis);
f = f(fm); % the inflection point
t = b(t);
f = b(f);
f = mean(vol(vol>f));
vol = clip(vol,t,f);

% Save some sample images for later
if 0
   slice = 226;
   vol_orig   = dip_image((squeeze(vol_orig  (:,:,slice))-50)*(255/205),'uint8');
   vol_smooth = dip_image((squeeze(vol_smooth(:,:,slice))-50)*(255/205),'uint8');
   vol_0      = dip_image((squeeze(vol       (:,:,slice))-50)*(255/205),'uint8');
   vol_32     = pathopening(vol,[],32,0,1);
   vol_32     = dip_image((squeeze(vol_32    (:,:,slice))-50)*(255/205),'uint8');
   vol_64     = pathopening(vol,[],64,0,1);
   vol_64     = dip_image((squeeze(vol_64    (:,:,slice))-50)*(255/205),'uint8');
   vol_128    = pathopening(vol,[],128,0,1);
   vol_128    = dip_image((squeeze(vol_128   (:,:,slice))-50)*(255/205),'uint8');
   vol_256    = pathopening(vol,[],256,0,1);
   vol_256    = dip_image((squeeze(vol_256   (:,:,slice))-50)*(255/205),'uint8');
   save sample_images vol*
end
clear vol_*

% Granulometry
L = unique(round(2.^(0:0.5:8.5))); % 2^8.5 pixels spans the whole image.
dis = zeros(size(L));
dis(1) = sum(vol);
for ii=2:length(L)
   disp(['L = ',num2str(L(ii))])
   tic;
   tmp = pathopening(vol,[],L(ii),0,1);
   toc
   dis(ii) = sum(tmp);
   clear tmp
end
save granulometry3d_results dis L

load granulometry3d_results
dis = dis-dis(1);
dis = dis/dis(end);

% Generate a plot with the length distribution
figure('position',[100,650,680,420])
ii = round(2*log2(L))/2;
ii(1) = 0.5;
xticklab = cell(size(ii));
for kk=2:2:length(ii)
   xticklab{kk} = num2str(1.4*round(2.^ii(kk)));
end
jj = (ii(2:end)+ii(1:end-1))/2;
h = bar(jj,diff(dis),1);
set(h,'facecolor',[0.7,0.7,0.7])
set(gca,'xtick',ii,'xticklabel',xticklab,'xlim',[ii(1)-0.2,ii(end)+0.2])
set(gca,'box','off','fontsize',14,'linewidth',1.5)
pos = get(gca,'position');
pos(3) = 1-2*pos(1);
set(gca,'position',pos);
ylabel('Relative density')
xlabel('Fibre length (\mum)')
h = axes('position',pos,'YAxisLocation','right','XAxisLocation','top','color','none');
hold on
plot(ii,dis,'b-','linewidth',2,'color',[0,0,0.7]);
set(h,'xtick',ii,'xticklabel','','xlim',[ii(1)-0.2,ii(end)+0.2],'ycolor',[0,0,0.7])
set(h,'fontsize',14,'linewidth',1.5)
ylabel('Cumulative distribution')
print -deps2c -r600 granulometry3d_graph.eps


% Write a slice from the volume at various scales, overlay sub-figure letter.
if 0

load sample_images
set(0,'DefaultFigureInvertHardcopy','off')

vol_orig
text(355,355,' (a) ','fontsize',24,'horizontalalignment','right','verticalalignment','bottom','margin',3,'background','black','color','white')
print -deps2c -r0 granulometry3d_orig.eps

(vol_0  -15)*(255/(91-15))
text(355,355,' (b) ','fontsize',24,'horizontalalignment','right','verticalalignment','bottom','margin',3,'background','black','color','white')
print -deps2c -r0 granulometry3d_000.eps

(vol_32 -15)*(255/(91-15))
text(355,355,' (c) ','fontsize',24,'horizontalalignment','right','verticalalignment','bottom','margin',3,'background','black','color','white')
print -deps2c -r0 granulometry3d_032.eps

(vol_64 -15)*(255/(91-15))
text(355,355,' (d) ','fontsize',24,'horizontalalignment','right','verticalalignment','bottom','margin',3,'background','black','color','white')
print -deps2c -r0 granulometry3d_064.eps

(vol_128-15)*(255/(91-15))
text(355,355,' (e) ','fontsize',24,'horizontalalignment','right','verticalalignment','bottom','margin',3,'background','black','color','white')
print -deps2c -r0 granulometry3d_128.eps

(vol_256-15)*(255/(91-15))
text(355,355,' (f) ','fontsize',24,'horizontalalignment','right','verticalalignment','bottom','margin',3,'background','black','color','white')
print -deps2c -r0 granulometry3d_256.eps

end
