%RIAOPENING   Used in ROTATIONINV and TIME_ACCURACY

%      RIAOPENING(IN,L,METHOD,STEPS) returns IN, closed using the
%      rotation-invariant, anisotropic morphology.
%      L is the  length of the structuring element.
%      METHOD, either 'discrete_line' or 'interpolated_line', selects
%      the line morphology method. 'discrete_line' is the default.
%      STEPS is the number of orientations used.

% Author:
% Cris Luengo, Pattern Recognitino Group, Delft University of Technology
% Copyright 2001.
%
% This code may be used freely for research purposes only. Do not
% copy or distribute without the prior written consent by the author.


function out = riaopening(in,l,method,steps)

% steps is the number of steps we divide the orientation in.
% steps = 8 means we have 8 orientations. 2*(steps-2) is the
% number of skewings that have to be performed.

% steps = ceil(pi*l) is the absolute maximum of steps needed.
% It is derived from the number of pixels at the edge of the region.
% This is ceil(pi*l/2) for the dilation
if nargin < 4
   steps = ceil(pi*l)*2;
end

if nargin > 2
   if ~ischar(method) || ~any(strcmp(method,{'interpolated_line','discrete_line'}))
      error('Illegal METHOD value');
   end
else
   method = 'discrete_line';
end

% Checks
if nargin < 2, error('Needs 2 parameters.'), end
in = dip_image(in);
if ndims(in) ~= 2, error('Only for 2D images.'), end
s = size(in);
if any(mod(s,2))
   error('Only even image sizes supported.')
end

% Stupid case:
if l<2
   out = in;
end

% Get angles [-pi/2,pi/2), remove 0rad from it.
angles = 0:(pi/steps):pi/2;
angles = angles(2:end);
angles = [-fliplr(angles),angles];
angles = angles(1:end-1);

out = dip_opening(in,[],[l,1],'rectangular'); % for angle=0
for angle=angles
   tmp = dip_opening(in,[],[l,angle],method);
   out = max(out,tmp);
end

% BTW: the orientation angle is implemented here with 0 rad being the vertical.
