%ROTATIONINV   Demo showing improved length estimation in a granulometry

% Author:
% Cris Luengo, Centre for Image Analysis, Uppsala, Sweden.
% Copyright 2008-2009.
%
% This code may be used freely for research purposes only. Do not
% copy or distribute without the prior written consent by the author.


img = create_image(0,40);
writeim(uint8(img*255),'rotationinv_0deg.tif','tiff',1)
img = create_image(pi/8,40);
writeim(uint8(img*255),'rotationinv_22deg.tif','tiff',1)
img = create_image(pi/4,40);
writeim(uint8(img*255),'rotationinv_45deg.tif','tiff',1)

% Granulometry
%L = 2.^(4:0.25:6.25);
L = 30:4:66;
phi = linspace(0,pi/4,8);
graU = zeros(length(phi),length(L));
graC = graU;
graR = graU;
for jj=1:length(phi);
   disp(['phi = ',num2str(phi(jj))])
   img = create_image(phi(jj),40);
   k0 = sum(img);
   k1 = 0-k0;
   for ii=1:length(L)
      disp(['L = ',num2str(L(ii))])
      rL = round(L(ii));
      graU(jj,ii) = sum(pathopening(img,[],rL,0,0));
      graC(jj,ii) = sum(pathopening(img,[],rL,0,1));
      graR(jj,ii) = sum(riaopening(img,L(ii),'interpolated_line'));
   end
   graU(jj,:) = graU(jj,:)-k0; graU(jj,:) = graU(jj,:)/k1;
   graC(jj,:) = graC(jj,:)-k0; graC(jj,:) = graC(jj,:)/k1;
   graR(jj,:) = graR(jj,:)-k0; graR(jj,:) = graR(jj,:)/k1;
   graR(jj,find(graR(jj,:)<0)) = 0; % interpolation errors
end
L = round(L*10)/10;
save rotationinv
%load rotationinv

figure('position',[100,100,600,900])
cm = zeros(8,3);
cm(:,3) = 0.7;
cm(:,2) = linspace(0,0.8,8);

subplot(3,1,1), hold on, box on
set(gca,'ColorOrder',cm)
set(gca,'fontsize',14)
plot([40,40],[0,1],'k-','color',[0.6,0.6,0.6])
h = plot(L,graU,'-o','linewidth',2);
set(gca,'xlim',[30,66],'xtick',L)
set(gca,'ylim',[0,1],'ytick',0:1/4:1)
legend(h([1,end]),{'0\circ','45\circ'},'location','northwest')
legend boxoff
ylabel('volume fraction')
%title('Path opening, unconstrained')
text(63.5,0.08,'(a)','fontsize',16,'fontweight','bold')

subplot(3,1,2), hold on, box on
set(gca,'ColorOrder',cm)
set(gca,'fontsize',14)
plot([40,40],[0,1],'k-','color',[0.6,0.6,0.6])
plot(L,graC,'-o','linewidth',2)
set(gca,'xlim',[30,66],'xtick',L)
set(gca,'ylim',[0,1],'ytick',0:1/4:1)
ylabel('volume fraction')
%title('Path opening, constrained')
text(63.5,0.08,'(b)','fontsize',16,'fontweight','bold')

subplot(3,1,3), hold on, box on
set(gca,'ColorOrder',cm)
set(gca,'fontsize',14)
plot([40,40],[0,1],'k-','color',[0.6,0.6,0.6])
plot(L,graR,'-o','linewidth',2)
set(gca,'xlim',[30,66],'xtick',L)
set(gca,'ylim',[0,1],'ytick',0:1/4:1)
xlabel('length (px)')
ylabel('volume fraction')
%title('RIA opening, interpolated lines')
text(63.5,0.08,'(c)','fontsize',16,'fontweight','bold')

print -deps2c -r600 rotationinv.eps
