%TIME_ACCURACY   Demo comparing various options for line openigs

% Author:
% Cris Luengo, Centre for Image Analysis, Uppsala, Sweden.
% Copyright 2008-2009.
%
% This code may be used freely for research purposes only. Do not
% copy or distribute without the prior written consent by the author.


results = [];
steps = round(2.^(2:0.5:9));
l = 50;
N = 50;

for jj=1:N

   a = make_testim/255;
   m = dilation(threshold(gradmag(a),'triangle'),3);
   results(jj).img = a;
   results(jj).mask = m;
   b = a; b(:) = 1;
   top = sqrt(mse(a,b,m));

   % Isotropic opening (upper limit in error)
   b = opening(a,2*sqrt(l/pi),'elliptic');
   results(jj).isotropicopening.rmse = sqrt(mse(a,b,m))/top;
   f = @() opening(a,2*sqrt(l/pi),'elliptic');
   results(jj).isotropicopening.time = timeit(f);

   % Path opening
   b = pathopening(a,[],l,0,0);
   results(jj).pathopening.rmse = sqrt(mse(a,b,m))/top;
   f = @() pathopening(a,[],l,0,0);
   results(jj).pathopening.time = timeit(f);

   % Path opening, constrained
   b = pathopening(a,[],l,0,1);
   results(jj).cpathopening.rmse = sqrt(mse(a,b,m))/top;
   f = @() pathopening(a,[],l,0,1);
   results(jj).cpathopening.time = timeit(f);

   % RIA openings, simple line SE
   for ii=1:length(steps);
      tic;
      b = riaopening(a,l,'discrete_line',steps(ii));
      results(jj).discrete_line.time(ii) = toc;
      results(jj).discrete_line.rmse(ii) = sqrt(mse(a,b,m))/top;
   end

   % RIA openings, line SE with interpolation
   for ii=1:length(steps);
      tic;
      b = riaopening(a,l,'interpolated_line',steps(ii));
      results(jj).interpolated_line.time(ii) = toc;
      results(jj).interpolated_line.rmse(ii) = sqrt(mse(a,b,m))/top;
   end

end

save time_accuracy results steps l

% Plot it all

load time_accuracy

timelim = log([0.025,20]);
timelab = [0.03,0.1,0.3,1,3,10];
timetick = log(timelab);
errlim = log([0.01,1]);
errlab = [0.01,0.03,0.1,0.3,1];
errtick = log(errlab);

figure('position',[100,350,800,650])
hold on, box on
set(gca,'xlim',timelim,'xtick',timetick,'xticklabel',timelab,...
        'ylim',errlim, 'ytick',errtick, 'yticklabel',errlab);
h = zeros(1,5);

x = []; y = [];
for ii=1:length(results)
   x = [x;results(ii).discrete_line.time];
   y = [y;results(ii).discrete_line.rmse];
end
h(1) = loglogplotstats(x,y,'s',2,[0,0,0.9]);

x = []; y = [];
for ii=1:length(results)
   x = [x;results(ii).interpolated_line.time];
   y = [y;results(ii).interpolated_line.rmse];
end
h(2) = loglogplotstats(x,y,'^',2,[0,0.5,0.9]);

x = []; y = [];
for ii=1:length(results)
   x = [x;results(ii).pathopening.time];
   y = [y;results(ii).pathopening.rmse];
end
h(3) = loglogplotstats(x,y,'o',2,[0.9,0.2,0]);

x = []; y = [];
for ii=1:length(results)
   x = [x;results(ii).cpathopening.time];
   y = [y;results(ii).cpathopening.rmse];
end
h(4) = loglogplotstats(x,y,'d',2,[0.9,0.5,0]);

x = []; y = [];
for ii=1:length(results)
   x = [x;results(ii).isotropicopening.time];
   y = [y;results(ii).isotropicopening.rmse];
end
h(5) = plot(log(mean(x)),log(mean(y)),'x','markersize',7,'linewidth',2,'color',[0,0,0.5])

set(gca,'fontsize',14,'linewidth',2);
xlabel('Execution time (s)')
ylabel('Root mean square error')
h = legend(h,'discrete lines','interpolated lines','path openings','constrained path openings','isotropic openings');
set(findobj(h,'type','line'),'linewidth',2,'markersize',7,'markerfacecolor','auto');
pos = get(h,'position');
pos(1) = pos(1)-0.02;
pos(2) = pos(2)-0.03;
set(h,'position',pos)

print -deps2c -r600 time_accuracy.eps

% Save a sample image from the test
if 0
   a = make_testim/255;
   dipshow(0,a,[0,1]);
   writeim(dip_image(a*255,'uint8'),'time_accuracy_input.tif','tiff',0,1)
end
